#version 150 compatibility
#define STEPS 8.0   // brightness steps [1.0 2.0 3.0 4.0 5.0 6.0 7.0 8.0 9.0 10.0 11.0 12.0 13.0 14.0 15.0 16.0 17.0 18.0]
#define CHROMA_ADD 0.20 // chroma increase [0.0 0.05 0.10 0.15 0.20 0.25 0.30]
#define SMOOTH // Smooth before filtering
#define HARDNESS 8.0 // edge hardness [1.0 2.0 4.0 8.0 16.0 32.0 64.0]
#define BACKGROUND 

uniform sampler2D colortex0;
uniform sampler2D depthtex0;

varying vec2 texcoord;
varying float cordy;

// From https://9to5answer.com/from-rgb-to-hsv-in-opengl-glsl under wtpfl
// All components are in the range [0…1], including hue.
vec3 rgb2hsv(vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

// All components are in the range [0…1], including hue.
vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}
// End code from answer

// Without smooting
#ifndef SMOOTH
float stepper(in float x)
{
    int value = int(STEPS * x + 0.5); //int between 0 and steps+0.5
    float ret = (float(value) - 0.5)  * (1.0/ STEPS);
    return ret;
}
#endif

const float zNear = 0.0001;
const float zFar = 1000.0;
float linearize_depth(float d)
{
    float z_n = 2.0 * d - 1.0;
    return 2.0 * zNear * zFar / (zFar + zNear - z_n * (zFar - zNear));
}

// With smooting
#ifdef SMOOTH
float stepper(in float x)
{
	float raw = STEPS * x;
    float value = int(raw + 0.5) - 0.5;
    float change = smoothstep(0.0, 1.0,  pow(raw - value, HARDNESS));
    if(value > raw) {
		change = change * -1.0;
    }
    float ret = (value + change)  * (1.0/ STEPS);
    return ret;
}
#endif

void main() {
    vec3 hsv = rgb2hsv(texture2D(colortex0, texcoord).rgb);
    float depth = texture2D(depthtex0, texcoord).r;
    
    if(depth != 1.0) { // Do not effect the sky
		hsv.z = stepper(hsv.z);
		
		if(hsv.y != 0) { // Make sure that white color is not distortet
			hsv.y = min(hsv.y + CHROMA_ADD, 1.0);
		}
		
#ifdef BACKGROUND
	float ldepth = linearize_depth(depth);
	if(ldepth > 0.2) {
		hsv.z = min(1.0, hsv.z + (ldepth - 0.2) * 0.05);
		hsv.y = max(0.0, hsv.y - (ldepth - 0.2) * 0.10);
	}
#endif
    }
    gl_FragData[0] = vec4(hsv2rgb(hsv), 1.0);
}
